package com.example.aped.ui.plots;

import android.net.sip.SipSession;
import android.content.DialogInterface;
import android.os.Bundle;
import androidx.fragment.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.Toast;
import com.example.aped.MainActivity;
import com.example.aped.R;
import com.example.aped.communication.Communicator;
import com.example.aped.utils.IXML;
import com.jjoe64.graphview.GraphView;
import com.jjoe64.graphview.series.DataPoint;
import com.jjoe64.graphview.series.LineGraphSeries;

import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import org.json.JSONException;
import org.json.JSONObject;
import java.util.ArrayList;
import java.util.List;

public class PlotFragment extends Fragment {

    /** Include the MainActivity.*/
    private MainActivity mainActivity;
    /** List of all the devices with buffers. **/
    private List<String> bufferedDevices;
    /** Line Chart View that is displayed **/
    private GraphView graphView;
    /** List of all the devices with buffers. **/
    private List<String> bufferedDevicesSelected;

    /**
     *                                                                                          paul
     * @param inflater
     * @param container
     * @param savedInstanceState
     * @return
     */
    @Override
    public View onCreateView(final LayoutInflater inflater,
                             final ViewGroup container,
                             final Bundle savedInstanceState) {
        this.mainActivity = (MainActivity) getActivity();
        this.bufferedDevices = getBufferedDevices(mainActivity.getXml());

        // Inflate the layout for this fragment
        View view =  inflater.inflate(R.layout.fragment_plot, container, false);
        this.graphView = view.findViewById(R.id.graph);

        getBufferedValues(mainActivity.getDelivery(), mainActivity.getXml());
        mainActivity.getConfigurationHandler().setCurrentView(R.id.nav_plots);

        Button buttonChoseDevice = view.findViewById(R.id.button_alert);
        buttonChoseDevice.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(final View v) {
                bufferedDevicesSelected = new ArrayList<>();
                MaterialAlertDialogBuilder alertDialogBuilder = new MaterialAlertDialogBuilder(mainActivity);
                alertDialogBuilder.setTitle("Choose your devices");
                alertDialogBuilder.setMultiChoiceItems(bufferedDevices.toArray(new String[bufferedDevices.size()]), null, new DialogInterface.OnMultiChoiceClickListener() {
                    @Override
                    public void onClick(final DialogInterface dialog, final int which, final boolean isChecked) {
                        if (isChecked) {
                            bufferedDevicesSelected.add(bufferedDevices.get(which));
                        } else if (bufferedDevices.contains(bufferedDevices.get(which))) {
                            bufferedDevicesSelected.remove(bufferedDevices.get(which));
                        }
                    }
                });
                alertDialogBuilder.setPositiveButton("display", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(final DialogInterface dialog, final int which) {
                        String data = "";
                        for (String item:bufferedDevicesSelected) {
                            data = data + " " + item;
                        }
                        Toast.makeText(mainActivity, data, Toast.LENGTH_SHORT).show();
                    }
                });
                alertDialogBuilder.create();
                alertDialogBuilder.show();
            }
        });

        return view;
    }

    private List<String> getBufferedDevices(final IXML xml) {
        List<String> returnList = new ArrayList<>();
        for (String deviceName : xml.getDeviceNames()) {
           if (xml.getBufferSize(deviceName) > 0) {
               returnList.add(deviceName);
           }
        }
        return returnList;
    }

    private void getBufferedValues(final Communicator communicator, final IXML xml) {
        for (String deviceName : this.bufferedDevices) {
            communicator.readBuffer(deviceName, response -> {
                try {
                    JSONObject buffer = response.getJSONObject("buffer");
                    for (String pin : (List<String>) (xml.getPort(deviceName)).get("pins")) {
                        String valueString = buffer.getString(pin);
                        valueString = valueString.substring(2, valueString.length() - 2);
                        String[] valueStringArray = valueString.split("', '");
                        int[] values = new int[valueString.length()];
                        for (int i = 0; i < valueStringArray.length; i++) {
                            values[i] = Integer.parseInt(valueStringArray[i]);
                        }

                        //display data
                        DataPoint[] dataPoints = new DataPoint[values.length];
                        for (int i = 0; i < values.length; i++){
                            dataPoints[i] = new DataPoint(i, values[i]);
                        }
                        LineGraphSeries<DataPoint> lineGraphSeries = new LineGraphSeries<>(dataPoints);
                        this.graphView.addSeries(lineGraphSeries);
                        this.graphView.getViewport().setMaxX(600);
                        this.graphView.getViewport().setMaxY(100);
                        this.graphView.getViewport().setXAxisBoundsManual(true);
                        this.graphView.getViewport().setYAxisBoundsManual(true);
                    }
                } catch (JSONException e) {
                    Log.e("PlotFragment", "error while parsing JSON on READ BUFFER: " + e.getMessage());
                }
            });
        }
    }
}
